<?php
/**
 * @package rgraph Joomla! Plugin
 * @version Version 1.2.5
 * @author  Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 * @copyright (C) 2016 Joachim Schmidt. All rights reserved.
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 *
 * change activity:
 *  01.10.2018: changed to namespaced classes (Joomla 3.x.API)
 *  08.08.2020: changed to support Jommla V4
 *  04.03.2021: changed to support Jommla V4 with bootstrap 5
 *  30.06.2021: corrected version checking to run on Joomal 3.10
 */

// no direct access
defined('_JEXEC') or die('Restricted access');


use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\String\StringHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Version;
use rgraph\plugin\plgContentrgraphHelper;

$lib = dirname(__FILE__) . '/lib/rgraph_chart.php';
require_once ($lib);

class plgContentrgraph extends CMSPlugin
{

    var $params;

    private $joomla_version;

    function __construct (&$subject, $params)
    {
        parent::__construct($subject, $params);
        $this->_plugin = PluginHelper::getPlugin('content', 'rgraph');
        $this->joomla_version = new Version();
    }

    function onContentPrepare ($context, &$row, &$params, $page = 0)
    {
        if (StringHelper::strpos($row->text, '{rgraph') === false)
            return true;
        else
        {
            $matches = array();
            $regex = "#{rgraph\\s*(.*?)}(.*?){/rgraph}#s";
            preg_match_all($regex, $row->text, $matches);
            $count = count($matches[0]);
            if ($count)
            {
                include_once 'helper.php';
                $this->helper = new plgContentrgraphHelper();

                $this->replaceContent($row, $matches, $count, $regex, $this->params);
                return true;
            }
            else
            {
                return true;
            }
        }
    }

    function addMediaResource ($name, $url, $type)
    {
        $document = Factory::getApplication()->getDocument();
        $base = URI::base(true) . "/";
        $joomlav4 = false;
        if (version_compare($this->joomla_version->getShortVersion(), '4.0', '>='))
        {
            $joomlav4 = true;
            $wa = Factory::getApplication()->getDocument()->getWebAssetManager();
        }
        switch ($type)
        {
            case "script":
                if (! $joomlav4)
                    $document->addScript($base . $url);
                else
                    $wa->registerAndUseScript($name, $url);
                break;

            case "style":
                if (! $joomlav4)
                    $document->addStyleSheet($base . $url);
                else
                    $wa->registerAndUseStyle($name, $url, [
                            'position' => 'after'
                    ], [], [
                            'webcomponent.joomla-alert'
                    ]);
                break;

            case "inlinescript":
                if (! $joomlav4)
                    $document->addScriptDeclaration($url);
                else
                    $wa->addInlineScript($url);
                break;

            case "inlinestyle":
                if (! $joomlav4)
                    $document->addStyleDeclaration($url);
                else
                    $wa->addInlineStyle($url);
                break;

            default:
                return;
        }
    }

    function replaceContent (&$row, $matches, $count, $regex, $params)
    {
        $error_string = null;
        $lang = Factory::getApplication()->getLanguage();
         $rc = $lang->load('plg_content_rgraph', JPATH_ADMINISTRATOR, null, true, true);
        if (! $rc)
        {
            $lang->load('plg_content_rgraph', JPATH_ADMINISTRATOR, "en-GB");
        }
        $plugin_parms = $params->toString();
        $plugin_parms = json_decode($plugin_parms, true);

        if (version_compare($this->joomla_version->getShortVersion(), '4', '>='))
            $joomla_version = 4;
        else
            $joomla_version = 3;

        for ($i = 0; $i < $count; $i ++)
        {
            $replace = "";
            $param_line = $this->helper->convert2Array($matches[0][$i]);
            $id = trim($matches[2][$i]);

            $result = $this->helper->getChartProperties($id, $plugin_parms, $param_line, $row);
            $error_string = $result['error'];
            $suppress_error = $result['suppress_error'];

            if (strlen($error_string) != null)
            {
                if ($result['debug'])
                    $replace = "<b>*** rgraph Parameter: </b><br /><pre>" . htmlspecialchars($matches[0][$i]) . "</pre>" . Text::_('INVALID_MSG') . $error_string;
                elseif (!$suppress_error)
                    $replace = "<br />" . Text::_('INVALID_MSG') . $error_string;
            }
            elseif ($result['popup'] != '')
            {
                HTMLHelper::_('bootstrap.modal');
                $url_css = 'media/plg_content_rgraph/css';
                $this->addMediaResource('rgraph.css', $url_css . '/rgraph.css', 'style');
                if (! isset($result['title']))
                    $result['title'] = $result['url'];
                $url = $result['url'];
                $width = $result['width'];
                $height = $result['height'];
                $replace .= $this->helper->buildModalhtml($id, $url, $result['popup'][0], $result['title'], $result['popup'][1], $width, $height);
            }
            else
            {
                $chart = $result['chart'];
                $options = $result['options'];
                $chart->set_options($options);

                $id = $chart->get_id();
                // $type = $chart->get_chartType();
                $width = $result['width'];
                $height = $result['height'];

                if (isset($result['draw_option']))
                    $draw_option = $result['draw_option'];
                else
                    $draw_option = ".draw();";

                if (isset($result['chart_alignment']))
                    $canvas_style = $result['chart_alignment'];
                if (isset($result['chart_script']))
                    $chart_script = $result['chart_script'];
                if (isset($result['event_script']))
                    $event_script = $result['event_script'];
                if (isset($result['canvas_class']))
                    $canvas_class = $result['canvas_class'];

                $url_base = 'media/plg_content_rgraph/js/';
                $url_css = 'media/plg_content_rgraph/css';

                if (isset($result['modal']))
                    $options->set_option("marginRight", $options->marginLeft);

                if ($result['debug'])
                {
                    $rgraph_json = $chart->toPrettyString();
                }
                else
                    $rgraph_json = $chart->toString();

                if ($rgraph_json == "")
                    $error_string = Text::_('ERROR_NO_JSON');

                if (! isset($canvas_class))
                    $canvas_class = "";

                if (! isset($result['chart_name']))
                    $chart_name = "chart" . $id;
                else
                    $chart_name = $result['chart_name'];

                $id = $chart->get_id();

                if (! isset($event_script))
                    $event_script = "";
                if (! isset($chart_script))
                    $chart_script = "";
                else
                    $chart_script = str_replace('&canvas', $id, $chart_script);

                $chart_type = $chart->get_chartType();

                $this->addMediaResource('rgraph.css', $url_css . '/rgraph.css', 'style');
                $this->addMediaResource('rgraph.common', $url_base . 'php-rgraph.common.js', 'script');
                $this->addMediaResource('rgraph.' . $chart->getScript($chart_type), $url_base . $chart->getScript($chart_type), 'script');

                if ($options->get_option('annotatable') == true)
                    $this->addMediaResource('rgraph.annotate', $url_base . $chart->getScript('annotate'), 'script');
                
                if ($result['drill_chart'] == "")
                {
                    $canvas = "\n<div style='padding-top: 25px;" . $canvas_style . "' class='" . $canvas_class . "'>";
                    $canvas .= "\n$error_string<canvas id='" . $id . "' width='" . $width . "' height='" . $height . "'>[No canvas support]></canvas>";
                    $canvas .= "\n</div>";
                }

                $bracket = "});";

                if ($result['combine_chart'] == false)
                {
                    if (isset($result['modal']))
                    {
                        HTMLHelper::_('bootstrap.modal');
                        if (! isset($result['title']))
                            $result['title'] = "modal chart";
                        $canvas = "<canvas id='" . $id . "' width='" . $width . "' height='" . $height . "' class='" . $canvas_class . "'>[No canvas support]></canvas>";
                        $replace .= $this->helper->buildModalhtml($id, $canvas, $result['modal'][0], $result['title'], $result['modal'][1], $width, null, $joomla_version);
                    }
                    elseif ($result['drill_chart'] == "")
                        $replace .= $canvas;
                }

                if ($result['combine_chart'] == true)
                {
                    if (! isset($result['chart_name']))
                        $chart_name = "combined" . $id;
                }

                if ($result['drill_chart'])
                {
                    $draw = $draw_option;
                    $draw_option = "";
                    $rgraph_script = "function " . $result['drill_chart'] . "(draw) { if (draw != false) { RGraph.reset('" . $id . "'); RGraph.redraw(); } ";
                    $bracket = " if (draw == false) return $chart_name; else { " . $chart_name . $draw . " } }";
                }
                elseif ($result['drill_charts'])
                {
                    $rgraph_script = "function " . $id . "_main() {  RGraph.reset('" . $id . "'); RGraph.redraw(); ";
                    $bracket = " }";
                    $rgraph_main = "jQuery(document).ready(function() { ";
                    $rgraph_main .= $id . "_main (); }); \n";

                    $this->addMediaResource('rgraph.main', $rgraph_main, 'inlinescript');
                }
                else
                {
                    $rgraph_script = "jQuery(document).ready(function() { ";
                }

                if (! empty($result['drill_tips']))
                    $chart_script .= "  " . $chart_name .
                            ".ontooltip = function (obj) { tips = obj.get('drill_tips'); var tooltip = RGraph.Registry.get('tooltip'); var index=tooltip.__index__;  if (tips[index] != undefined && typeof window[ tips[index] ] == 'function') window[ tips[index] ](false).draw(); else RGraph.hideTooltip(); }";

                if (! empty($result['drill_charts']))
                    $chart_script .= "  " . $chart_name . ".onclick = function (e, shape) { var obj = e.target.__object__; charts = " . $chart_name . ".get('drill_charts'); var index=shape.index;  if (charts[index] != undefined && typeof window[ charts[index] ] == 'function') { RGraph.reset('" . $id .
                            "'); window[ charts[index] ]();  } }\n  " . $chart_name . ".onmousemove = function (e, shape) {  e.target.style.cursor = 'pointer'; return true; }";

                $rgraph_script .= $event_script . "\n  var " . $chart_name . " = new RGraph." . $chart_type . "( " . $rgraph_json . " )" . $draw_option . "\n" . $chart_script . "  " . $bracket . "\n";

                $this->addMediaResource('rgraph.script', $rgraph_script, 'inlinescript');
            }

            $row->text = str_replace($matches[0][$i], $replace, $row->text);

            $chart_script = "";
            $event_script = "";
            $result = array();
            $error_string = null;
        }
        return true;
    }
}
